@@ -36,7 +36,7 @@ extern bool secret_buffer_charset_test_byte(const secret_buffer_charset *cset, U
 /* Test whether the charset contains a unicode character.  This uses the perl regex
  * engine if the codepoint is higher than 0x7F, to ensure correct matching.
  */
-extern bool secret_buffer_charset_test_codepoint(const secret_buffer_charset *cset, uint32_t cp);
+extern bool secret_buffer_charset_test_codepoint(const secret_buffer_charset *cset, U32 cp);
 
 /* encoding flags can be combined with other flags */
 #define SECRET_BUFFER_ENCODING_MASK      0xFF
@@ -110,6 +110,7 @@ extern bool secret_buffer_match_bytestr(secret_buffer_parse *p, char *data, size
  */
 extern SSize_t secret_buffer_sizeof_transcode(secret_buffer_parse *src, int dst_encoding);
 extern bool secret_buffer_transcode(secret_buffer_parse *src, secret_buffer_parse *dst);
+extern bool secret_buffer_copy_to(secret_buffer_parse *src, SV *dst_sv, int encoding, bool append);
 
 /* Create a new Crypt::SecretBuffer object with a mortal ref and return its secret_buffer
  * struct pointer.
@@ -117,7 +118,8 @@ extern bool secret_buffer_transcode(secret_buffer_parse *src, secret_buffer_pars
  * FREETMPS destroys the ref which destroys the object which destroys the magic which destroys
  * the secret_buffer struct which also clears it.
  * If you supply a pointer to receive ref_out, you can then increment the refcount or copy the
- * ref to a new SV if you want to keep the object.
+ * ref to a new SV if you want to keep the object.  As a weak-ref, it's also convenient to push
+ * onto perl's stack.
  * Always returns a secret_buffer, or croaks on failure.
  */
 extern secret_buffer * secret_buffer_new(size_t capacity, SV **ref_out);
